--- ./src/kbd.c.orig	2013-10-28 09:44:15.000000000 +0900
+++ ./src/kbd.c	2016-09-11 15:50:30.000000000 +0900
@@ -25,6 +25,7 @@
 
 #include "xf86.h"
 #include "atKeynames.h"
+#include "xf86Priv.h"
 #include "xf86Privstr.h"
 
 #include <X11/extensions/XI.h>
@@ -100,6 +101,16 @@
     NULL
 };
 
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 12
+static const char *kbd98Defaults[] = {
+    "Protocol",                "standard",
+    "XkbRules",                "xfree98",
+    "XkbModel",                "pc98",
+    "XkbLayout",       "jp",
+    NULL
+};
+#endif
+
 static char *xkb_rules;
 static char *xkb_model;
 static char *xkb_layout;
@@ -162,7 +173,13 @@
     pInfo->fd = -1;
     pInfo->dev = NULL;
 
-    defaults = kbdDefaults;
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 12
+    if (xf86IsPc98())
+       defaults = kbd98Defaults;
+    else
+#endif
+       defaults = kbdDefaults;
+
     xf86CollectInputOptions(pInfo, defaults
 #if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 12
             , NULL
@@ -400,6 +417,14 @@
       RemoveEnabledDevice(pInfo->fd);
     pKbd->KbdOff(pInfo, what);
     device->public.on = FALSE;
+	/*
+	 * Close device file for keyboards which are not attached
+	 * to console, otherwise they can't be opened again after
+	 * relogin when using session manager like xdm.
+	 * X server will take care about console attached keyboards.
+	 */
+	if (pInfo->fd != xf86Info.consoleFd)
+		close(pInfo->fd);
     break;
 
   default:
@@ -442,12 +467,18 @@
    * physical keyboard key.
    */
 
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 12
+  if (!xf86IsPc98()) {
+#endif
   state = XkbStateFieldFromRec(&keyc->xkbInfo->state);
 
   if (((state & AltMask) == AltMask) && (scanCode == KEY_SysReqest))
     scanCode = KEY_Print;
   else if (scanCode == KEY_Break)
     scanCode = KEY_Pause;
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 12
+  }
+#endif
 
   xf86PostKeyboardEvent(device, scanCode + MIN_KEYCODE, down);
 }
